//////////////////////////////////////////////////////////////////////
//
// Project : BlueEye
//
// Autor : Bernard Lidicky
//
// Jpg.cpp: implementation of the CJPEG class.
//
//////////////////////////////////////////////////////////////////////

#include <stdafx.h>
#include "Jpg.h"

extern "C" 
{
	#include "jpeglib\\jpeglib.h"
	#include <setjmp.h>
}


//////////////////////////////////////////////////////////////////////
// error handler

struct my_error_mgr
{
	struct	jpeg_error_mgr pub;
	jmp_buf setjmp_buffer; 
};

typedef struct my_error_mgr *my_error_ptr;

METHODDEF(void) my_error_exit( j_common_ptr cinfo );

//////////////////////////////////////////////////////////////////////
// Obsluhuje fatal errors

METHODDEF(void) my_error_exit( j_common_ptr cinfo )
{
    my_error_ptr  myerr = (my_error_ptr) cinfo->err;
    CHAR          buffer[JMSG_LENGTH_MAX];
   
    (*cinfo->err->format_message) (cinfo, buffer);
    MessageBox( NULL, buffer, "Pri nacitani nastla chyba!", MB_ICONSTOP );
    longjmp(myerr->setjmp_buffer, 1);
}


////////////////////////////////////////////////////////////////////////////
// Konstruktor

CJPEG::CJPEG() {}


////////////////////////////////////////////////////////////////////////////
// Destruktor

CJPEG::~CJPEG() {}


////////////////////////////////////////////////////////////////////////////
// Nacte JPG soubor

BYTE *CJPEG::LoadJPEG( LPSTR fileName, UINT *width, UINT *height, INT *Bits )
{
	struct			jpeg_decompress_struct cinfo;
    struct			my_error_mgr jerr;
    FILE			*infile=NULL;       
    JSAMPARRAY		buffer;	
    BYTE			*dataBuf;


	// First open the file...
    infile = fopen(fileName, "rb");
	 if( infile == NULL ) {
		 TRACE( "Couldn't open JPEG-File!" );
		 return NULL;
	 }

    // We set up the normal JPEG error routines, then override error_exit...
    cinfo.err = jpeg_std_error(&jerr.pub);
    jerr.pub.error_exit = my_error_exit;

    // Establish the setjmp return context for my_error_exit to use...
    if( setjmp(jerr.setjmp_buffer) )
	{
		jpeg_destroy_decompress(&cinfo);
        if( infile != NULL ) fclose(infile);
        return NULL;
	}

    // Now we can initialize the JPEG decompression object...
    jpeg_create_decompress(&cinfo);

    // Tell JPEG Lib that we get the data from a file...
    jpeg_stdio_src(&cinfo, infile);

    // Now read the Header...
    jpeg_read_header(&cinfo, TRUE);

    // Start the decompressor...
    jpeg_start_decompress(&cinfo);

	// Compute WidthBytes for 24bit...
	INT WidthBytes = cinfo.output_width*3;
	if(WidthBytes & 0x003) WidthBytes = (WidthBytes | 3) + 1;

	// Alloc Memory to hold the decompressed pic...
	dataBuf = new BYTE[WidthBytes * cinfo.output_height];

    // how big is this thing gonna be?
    *width  = cinfo.output_width;
    *height = cinfo.output_height;
	*Bits   = cinfo.out_color_components << 3;

    // Make a one-row-high sample array that will go away when done with image...
    buffer = (*cinfo.mem->alloc_sarray) ((j_common_ptr) &cinfo, JPOOL_IMAGE, WidthBytes, 1);

	// Now read row by row the data into our buffer...
    if (cinfo.out_color_components==3) // and RGB Image...
	{
		INT OurRow = (cinfo.output_height-1)*WidthBytes;
		while (cinfo.output_scanline < cinfo.output_height)
		{
			// Read Scanline...
			jpeg_read_scanlines(&cinfo, buffer, 1);

			// Write Scanline, plus RGB->BGR and flipping...
			INT		col = 0;
			BYTE	*jpegline = buffer[0];
			for( UINT count=0; count<cinfo.output_width; count++, col += 3 ) 
			{
				dataBuf[OurRow + col + 0] = *(jpegline + col + 2); 
				dataBuf[OurRow + col + 1] = *(jpegline + col + 1);
				dataBuf[OurRow + col + 2] = *(jpegline + col + 0);
			}
			OurRow -= WidthBytes;
		}
	}
	else // Its a Grayscale one...
	{
		INT OurRow = (cinfo.output_height-1)*WidthBytes;
		while (cinfo.output_scanline < cinfo.output_height)
		{
			// Read Scanline...
			jpeg_read_scanlines(&cinfo, buffer, 1);

			// Write Scanline, plus flipping...
			INT		col = 0;
			BYTE	iGray = 0;
			BYTE	*jpegline = buffer[0];
			for( UINT count=0; count<cinfo.output_width; count++, col+=3 ) 
			{
		        iGray = *(jpegline + count);
				dataBuf[OurRow + col + 0] = iGray; 
				dataBuf[OurRow + col + 1] = iGray;
				dataBuf[OurRow + col + 2] = iGray;
			}
			OurRow -= WidthBytes;
		}
	}

    jpeg_finish_decompress(&cinfo);
    jpeg_destroy_decompress(&cinfo);
    fclose(infile);

	// Yeah...
	return dataBuf;
}


